@@ -1,12 +1,12 @@
 #
-# $Id: Factor.pm,v 0.04 2004/01/06 13:59:18 sts Exp $
+# $Id: Factor.pm,v 0.05 2004/01/07 23:56:19 sts Exp $
 
 package Math::Factor;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use integer;
-use strict;
+use strict 'vars';
 use warnings;
 
 our ($format_factors, $format_match_number,
@@ -18,8 +18,8 @@ use base qw(Exporter);
 our (@EXPORT_OK, %EXPORT_TAGS, @subs_factor,
      @subs_match, @subs);
 
-@subs_factor = qw(factor show_factor);
-@subs_match = qw(match show_match);
+@subs_factor = qw(factor each_factor show_factor);
+@subs_match = qw(match each_match show_match);
 @subs = (@subs_factor, @subs_match);
 
 @EXPORT_OK = @subs;
@@ -70,12 +70,12 @@ be accessed by @{$factors{9}}.
 =cut
 
 sub factor {
-    my $number_ref = $_[0];
+    my $data_ref = $_[0];
     croak q~usage: factor (\@numbers)~
-      unless ref $number_ref eq 'ARRAY' && @$number_ref;
+      unless ref $data_ref eq 'ARRAY' && @$data_ref;
 
     my %factor;
-    foreach (@$number_ref) {
+    foreach (@$data_ref) {
         for (my $i = 1; $i <= $_; $i++) {
             if ($_ % $i == 0)  {
                 push @{$factor{$_}}, $i;
@@ -101,14 +101,14 @@ ones by $matches{9}[1][0] and $matches{9}[1][1], and so on.
 =cut
 
 sub match {
-    my $number_ref = $_[0];
+    my $data_ref = $_[0];
     croak q~usage: match (\%factors)~
-      unless ref $number_ref eq 'HASH' && %$number_ref;
+      unless ref $data_ref eq 'HASH' && %$data_ref;
 
     my ($i, %match);
-    foreach (keys %$number_ref) {
+    foreach (keys %$data_ref) {
         $i = 0;
-        my @cmp = my @base = @{$$number_ref{$_}};
+        my @cmp = my @base = @{$$data_ref{$_}};
         foreach my $base (@base) {
             foreach my $cmp (@cmp) {
                 if ($cmp >= $base) {
@@ -125,6 +125,60 @@ sub match {
     return %match;
 }
 
+=head2 each_factor
+
+Returns each factor of a number in a scalar context.
+
+ while ($factor = each_factor(\%factors, $numbers[0])) {
+     print "$factor\n";
+ }
+
+=cut
+
+sub each_factor {
+    my ($data_ref, $number) = @_;
+    croak q~usage: each_factor (\%factors, $number)~
+      unless ref $data_ref eq 'HASH' && $number;
+
+    unless (${__PACKAGE__.'::data'}) {
+        @{__PACKAGE__.'::data'} = @{$$data_ref{$number}};
+        ${__PACKAGE__.'::data'} = 1;
+    }
+
+    if (@{__PACKAGE__.'::data'}) {
+        return shift @{__PACKAGE__.'::data'};
+    }
+    else { ${__PACKAGE__.'::data'} = 0; return }
+}
+
+=head2 each_match
+
+Returns each match of a number in array context.
+
+ while (@match = each_match(\%matches, $numbers[0])) {
+     print "$numbers[0] == $match[0] * $match[1]\n";
+ }
+
+=cut
+
+sub each_match {
+    my ($data_ref, $number) = @_;
+    croak q~usage: each_match (\%matches, $number)~
+      unless ref $data_ref eq 'HASH' && $number;
+
+    unless (${__PACKAGE__.'::data'}) {
+        @{__PACKAGE__.'::data'} = @{$$data_ref{$number}};
+        ${__PACKAGE__.'::data'} = 1;
+    }
+
+    if (wantarray && @{__PACKAGE__.'::data'}) {
+        my @match = (${__PACKAGE__.'::data'}[0][0], ${__PACKAGE__.'::data'}[0][1]);
+        splice (@{__PACKAGE__.'::data'}, 0, 1);
+        return @match;
+    }
+    else { ${__PACKAGE__.'::data'} = 0; return }
+}
+
 =head2 show_factor
 
 Outputs all numbers and their according factors to STDOUT.
@@ -138,9 +192,9 @@ Outputs all numbers and their according factors to STDOUT.
 =cut
 
 sub show_factor {
-    my $number_ref = $_[0];
+    my $data_ref = $_[0];
     croak q~usage: show_factors (\%factors)~
-      unless ref $number_ref eq 'HASH' && %$number_ref;
+      unless ref $data_ref eq 'HASH' && %$data_ref;
 
     print <<'EOT';
 -------
@@ -153,7 +207,7 @@ EOT
     eval $format_factors;
     croak $@ if $@;
 
-    foreach (sort {$a <=> $b} keys %$number_ref) {
+    foreach (sort {$a <=> $b} keys %$data_ref) {
         $ul = '-' x length;
         write;
     }
@@ -173,9 +227,9 @@ Outputs all numbers and their according matching multiplications to STDOUT.
 =cut
 
 sub show_match {
-    my $number_ref = $_[0];
+    my $data_ref = $_[0];
     croak q~usage: show_match (\%match)~
-      unless ref $number_ref eq 'HASH' && %$number_ref;
+      unless ref $data_ref eq 'HASH' && %$data_ref;
 
     print <<'EOT';
 -------
@@ -185,7 +239,7 @@ MATCHES
 EOT
 
     no warnings;
-    foreach (sort {$a <=> $b} keys %$number_ref) {
+    foreach (sort {$a <=> $b} keys %$data_ref) {
         my $ul = '-' x length;
 
         eval $format_match_number;
@@ -196,7 +250,7 @@ EOT
         eval $format_match_matches;
         croak $@ if $@;
 
-        for ($i = 0; $$number_ref{$_}[$i]; $i++) { write }
+        for ($i = 0; $$data_ref{$_}[$i]; $i++) { write }
 
         print <<'EOT';
 
@@ -212,7 +266,7 @@ $_
 @<<<<<<<<<<<<<<<<<<<<<<<<<
 $ul
 @<<<<<<<<<<<<<<<<<<<<<<<<<
-"@{$$number_ref{$_}}"
+"@{$$data_ref{$_}}"
 
 
 .
@@ -230,28 +284,24 @@ $ul
 $format_match_matches = '
     format =
 @<<<<<<<<<<<* @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
-$$number_ref{$_}[$i][0] $$number_ref{$_}[$i][1]
+$$data_ref{$_}[$i][0] $$data_ref{$_}[$i][1]
 .
 ';
 
 1;
 __END__
 
-=head2 LIMITATIONS
-
-Evaluating ranges of factors or multiplication matches is unimplemented aswell as iterating over factors or multiplication matches.
-
 =head2 EXPORT
 
-C<factor(), match(), show_factor(), show_match()> upon request.
+C<factor(), match(), each_factor(), each_match(), show_factor(), show_match()> upon request.
 
 B<TAGS>
 
 C<:all - *()>
 
-C<:factor - factor(), show_factor()>
+C<:factor - factor(), each_factor(), show_factor()>
 
-C<:match - match(), show_match()>
+C<:match - match(), each_match(), show_match()>
 
 =head1 SEE ALSO
 
